package space.kklochko.jdbc_hospital_example.db.repositories

import groovy.sql.Sql
import space.kklochko.jdbc_hospital_example.config.factories.LoadDataBaseConfigFromEnvFile
import space.kklochko.jdbc_hospital_example.config.models.DataBaseConfig
import space.kklochko.jdbc_hospital_example.db.factories.DataBaseConnection
import space.kklochko.jdbc_hospital_example.models.Indicator
import spock.lang.Shared
import spock.lang.Specification
import spock.lang.Stepwise
import spock.lang.Subject

import java.sql.Timestamp
import java.util.UUID

@Stepwise
class IndicatorRepositorySpec extends Specification {
    @Shared
    Sql sql

    @Shared
    DataBaseConfig db

    @Shared
    DataBaseConnection dbc

    def setupSpec() {
        db = (new LoadDataBaseConfigFromEnvFile()).load("db.testing")
        dbc = new DataBaseConnection(db)

        IndicatorRepository repo = new IndicatorRepository(dbc)
        repo.createTable()

        sql = Sql.newInstance(db.getUrl(), db.getProperties().get("user"), db.getProperties().get("password"))

        sql.execute("insert into indicators values " +
               "('bcbbcdb4-702c-11ee-9113-c0e4349366ab', 'Samples', 'fine', '2023-10-17 16:30:00')," +
               "('becbafac-702c-11ee-9113-c0e4349366ab', 'Some samples', 'well', '2023-10-17 11:30:00')")

    }

    def cleanupSpec() {
        sql.execute("delete from indicators")
    }

    def "Read one indicator"() {
        given: "I have an existing indicator"
        UUID id = UUID.fromString("becbafac-702c-11ee-9113-c0e4349366ab")

        and: "I have a repo"
        @Subject
        IndicatorRepository repo = new IndicatorRepository(dbc)

        when: "reading the entry"
        def result = repo.read(id)

        then: "checking that the reading was successful"
        result

        and: "checking the entry with the expected data"
        def data = sql.firstRow("SELECT type, values, timestamp FROM indicators WHERE id = ?", id)
        with(data) {
            get("type") == 'Some samples'
            get("values") == 'well'
            get("timestamp") == Timestamp.valueOf('2023-10-17 11:30:00')
        }
    }

    def "Read all indicators"() {
        given: "I have a repo"
        @Subject
        IndicatorRepository repo = new IndicatorRepository(dbc)

        when: "reading the entries"
        def result = repo.readAll()

        then: "checking that the reading was successful"
        result

        and: "checking the entries with the expected data"
        def data = sql.rows("SELECT type, values, timestamp FROM indicators")
        data.collect{it.get("type")} == ['Samples', 'Some samples']
    }

    def "Add one indicator"() {
        given: "I create an indicator"
        Timestamp timestamp = Timestamp.valueOf("2023-10-17 14:30:00")
        Indicator indicator = new Indicator("Blood", "Fine", timestamp)

        and: "I have a repo"
        @Subject
        IndicatorRepository repo = new IndicatorRepository(dbc)

        when: "inserting the entry"
        def result = repo.create(indicator)

        then: "checking that the insert was successful"
        result
        indicator.id

        and: "checking that the entry is added"
        def isExist = sql.firstRow("SELECT type, values, timestamp FROM indicators WHERE id = ?", indicator.id)
        isExist
    }

    def "Update one indicator"() {
        given: "I have an existing indicator"
        UUID id = UUID.fromString("becbafac-702c-11ee-9113-c0e4349366ab")
        Timestamp timestamp = Timestamp.valueOf("2023-10-18 14:30:00")
        Indicator indicator = new Indicator(id, "Blood2", "Fine2", timestamp)

        and: "I have a repo"
        @Subject
        IndicatorRepository repo = new IndicatorRepository(dbc)

        when: "updating the entry"
        def result = repo.update(indicator)

        then: "checking that the update was successful"
        result

        and: "checking that the entry is updated"
        def data = sql.firstRow("SELECT type, values, timestamp FROM indicators WHERE id = ?", indicator.id)
        with(data) {
            get("type") == indicator.type
            get("values") == indicator.values
            get("timestamp") == indicator.timestamp
        }
    }

    def "Remove one indicator"() {
        given: "I have the id of an existing indicator"
        UUID id = UUID.fromString("bcbbcdb4-702c-11ee-9113-c0e4349366ab")

        and: "I have a repo"
        @Subject
        IndicatorRepository repo = new IndicatorRepository(dbc)

        when: "deleting the entry"
        def result = repo.delete(id)

        then: "checking that the delete was successful"
        result

        and: "checking that the entry was deleted"
        def isExist = sql.firstRow("SELECT type, values, timestamp FROM indicators WHERE id = ?", id)
        !isExist
    }
}
